@@ -13,6 +13,7 @@ import os
 import shutil
 import subprocess
 import sys
+import timeit
 from functools import reduce
 from importlib import reload
 from itertools import cycle
@@ -26,6 +27,7 @@ from pymtl3.passes.BasePass import BasePass
 from pymtl3.passes.rtlir import RTLIRDataType as rdt
 from pymtl3.passes.rtlir import RTLIRGetter
 from pymtl3.passes.rtlir import RTLIRType as rt
+from pymtl3.passes.rtlir.rtype.RTLIRDataType import _get_rtlir_dtype_struct
 
 from ..errors import VerilogImportError
 from ..util.utility import (
@@ -60,6 +62,13 @@ class VerilogVerilatorImportPass( BasePass ):
   #: Default value: ``False``
   verbose             = MetadataKey(bool)
 
+  #: Spend more time on compilation for faster simulation performance
+  #:
+  #: Type: ``bool``; input
+  #:
+  #: Default value: ``False``
+  fast                = MetadataKey(bool)
+
   #: Use Verilog ``line_trace`` output as the Python line_trace return value.
   #:
   #: Type: ``bool``; input
@@ -388,8 +397,10 @@ class VerilogVerilatorImportPass( BasePass ):
       try:
         ip_cfg.vprint(f"Verilating {ip_cfg.translated_top_module} with command:", 2)
         ip_cfg.vprint(f"{cmd}", 4)
+        t0 = timeit.default_timer()
         subprocess.check_output(
             cmd, stderr = subprocess.STDOUT, shell = True )
+        ip_cfg.vprint(f"verilate time: {timeit.default_timer()-t0}")
       except subprocess.CalledProcessError as e:
         succeeds = False
         err_msg = e.output if not isinstance(e.output, bytes) else \
@@ -425,6 +436,8 @@ class VerilogVerilatorImportPass( BasePass ):
     wrapper_name = ip_cfg.get_c_wrapper_path()
     verilator_xinit_value = ip_cfg.get_vl_xinit_value()
     verilator_xinit_seed = ip_cfg.get_vl_xinit_seed()
+    has_clk = int(ph_cfg.has_clk)
+
     ip_cfg.vprint("\n=====Generate C wrapper=====")
 
     # Generate port declarations for the verilated model in C
@@ -471,12 +484,14 @@ class VerilogVerilatorImportPass( BasePass ):
       try:
         ip_cfg.vprint("Compiling shared library with command:", 2)
         ip_cfg.vprint(f"{cmd}", 4)
+        t0 = timeit.default_timer()
         subprocess.check_output(
             cmd,
             stderr = subprocess.STDOUT,
             shell = True,
             universal_newlines = True
         )
+        ip_cfg.vprint(f"shared library compilation time: {timeit.default_timer()-t0}")
       except subprocess.CalledProcessError as e:
         succeeds = False
         err_msg = e.output if not isinstance(e.output, bytes) else \
@@ -613,9 +628,9 @@ class VerilogVerilatorImportPass( BasePass ):
   def serialize_cfg( s, ip_cfg ):
     d = {}
     s._volatile_configs = [
+      'verilog_hash',
       'vl_line_trace', 'vl_coverage', 'vl_line_coverage', 'vl_toggle_coverage',
-      'vl_mk_dir', 'vl_enable_assert', 'vl_opt_level',
-      'vl_unroll_count', 'vl_unroll_stmts',
+      'vl_mk_dir', 'vl_enable_assert',
       'vl_W_lint', 'vl_W_style', 'vl_W_fatal', 'vl_Wno_list',
       'vl_xinit', 'vl_trace',
       'vl_trace_timescale', 'vl_trace_cycle_time',
@@ -631,6 +646,8 @@ class VerilogVerilatorImportPass( BasePass ):
   def is_same_cfg( s, prev, new ):
     _volatile_configs = copy.copy(s._volatile_configs)
     _volatile_configs.append('ImportPassName')
+    if not all(cfg in prev and cfg in new for cfg in _volatile_configs):
+      return False
     return all(prev[cfg] == new[cfg] for cfg in _volatile_configs)
 
   #-------------------------------------------------------------------------
@@ -754,10 +771,21 @@ m->{name}{sub} = {deference}model->{name}{sub};
 
         def _get_arg_str( name, obj ):
           # Support common python types and Bits/BitStruct
+
           if isinstance( obj, (int, bool, str) ):
             return str(obj)
-          elif obj == None:
+
+          elif obj is None:
             return 'None'
+
+          elif isinstance( obj, type ) and issubclass( obj, Bits ):
+            nbits = obj.nbits
+            Bits_name = f"Bits{nbits}"
+            if Bits_name not in symbols and nbits >= 256:
+              Bits_class = mk_bits( nbits )
+              symbols.update( { Bits_name : Bits_class } )
+            return Bits_name
+
           elif isinstance( obj, Bits ):
             nbits = obj.nbits
             value = int(obj)
@@ -767,6 +795,23 @@ m->{name}{sub} = {deference}model->{name}{sub};
               Bits_class = mk_bits( nbits )
               symbols.update( { Bits_name : Bits_class } )
             return Bits_arg_str
+
+          elif is_bitstruct_class(obj):
+            # Shunning: get the unique name of bitstruct class
+            BitStruct_name = _get_rtlir_dtype_struct( obj() ).get_name()
+
+            if BitStruct_name in symbols:
+              assert symbols[BitStruct_name] is obj, "Same bitstruct name but different classes!"
+            else:
+              symbols.update( { BitStruct_name : obj } )
+
+            return BitStruct_name
+
+          # FIXME formalize this
+          elif isinstance( obj, type ) and hasattr( obj, 'req' ) and hasattr( obj, 'resp' ):
+            symbols.update( { obj.__name__ : obj } )
+            return obj.__name__
+
           elif is_bitstruct_inst( obj ):
             raise TypeError("Do you really want to pass in an instance of "
                             "a BitStruct? Contact PyMTL developers!")
@@ -781,22 +826,7 @@ m->{name}{sub} = {deference}model->{name}{sub};
             # if bs_name not in symbols:
               # symbols.update( { bs_name : obj } )
             # return bs_name
-          elif isinstance( obj, type ) and issubclass( obj, Bits ):
-            nbits = obj.nbits
-            Bits_name = f"Bits{nbits}"
-            if Bits_name not in symbols and nbits >= 256:
-              Bits_class = mk_bits( nbits )
-              symbols.update( { Bits_name : Bits_class } )
-            return Bits_name
-          elif is_bitstruct_class(obj):
-            BitStruct_name = obj.__name__
-            if BitStruct_name not in symbols:
-              symbols.update( { BitStruct_name : obj } )
-            return BitStruct_name
-          # FIXME formalize this
-          elif isinstance( obj, type ) and hasattr( obj, 'req' ) and hasattr( obj, 'resp' ):
-            symbols.update( { obj.__name__ : obj } )
-            return obj.__name__
+
           raise TypeError( f"Interface constructor argument {obj} is not an int/Bits/BitStruct/TypeTuple!" )
 
         name, cls = ifc.get_name(), ifc.get_class()
@@ -890,7 +920,8 @@ m->{name}{sub} = {deference}model->{name}{sub};
 
   def gen_port_vector_input( s, lhs, rhs, mangled_rhs, dtype, symbols ):
     dtype_nbits = dtype.get_length()
-    blocks   = [ f's.{mangled_rhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
+    blocks   = [ '',
+                 f's.{mangled_rhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
                  '@update',
                  f'def isignal_{mangled_rhs}():',
                  f'  s.{mangled_rhs} @= {rhs}' ]
@@ -904,15 +935,11 @@ m->{name}{sub} = {deference}model->{name}{sub};
     if dtype_name not in symbols:
       symbols[dtype_name] = dtype.get_class()
 
-    blocks   = [ f's.{mangled_rhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
+    blocks   = [ '',
+                 f's.{mangled_rhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
                  '@update',
-                 f'def istruct_{mangled_rhs}():' ]
-
-    # We write each struct field to tmp
-    upblk_content, pos = s._gen_struct_write( 'i', rhs, 's.'+mangled_rhs, dtype, 0 )
-    assert pos == dtype_nbits
-    make_indent( upblk_content, 1 )
-    blocks += upblk_content
+                 f'def istruct_{mangled_rhs}():',
+                 f'  s.{mangled_rhs} @= {rhs}' ]
 
     # We don't create a new struct if we are copying values from pymtl
     # land to verilator, i.e. this port is the input to the imported
@@ -987,7 +1014,8 @@ m->{name}{sub} = {deference}model->{name}{sub};
 
   def gen_port_vector_output( s, lhs, mangled_lhs, rhs, dtype, symbols ):
     dtype_nbits = dtype.get_length()
-    blocks   = [ f's.{mangled_lhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
+    blocks   = [ '',
+                 f's.{mangled_lhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
                  '@update',
                  f'def osignal_{mangled_lhs}():',
                  f'  {lhs} @= s.{mangled_lhs}' ]
@@ -998,11 +1026,12 @@ m->{name}{sub} = {deference}model->{name}{sub};
   def gen_port_struct_output( s, lhs, mangled_lhs, rhs, dtype, symbols ):
     dtype_nbits = dtype.get_length()
     # If the top-level signal is a struct, we add the datatype to symbol?
-    dtype_name = dtype.get_class().__name__
+    dtype_name = dtype.get_name()
     if dtype_name not in symbols:
       symbols[dtype_name] = dtype.get_class()
 
-    blocks   = [ f's.{mangled_lhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
+    blocks   = [ '',
+                 f's.{mangled_lhs} = Wire( {s._gen_bits_decl(dtype_nbits)} )',
                  '@update',
                  f'def ostruct_{mangled_lhs}():' ]
 
@@ -1012,11 +1041,10 @@ m->{name}{sub} = {deference}model->{name}{sub};
     # We create a new struct if we are copying values from verilator
     # world to pymtl land and send it out through the output of this
     # component
-    upblk_content = [ f"{lhs} @= {dtype_name}()" ]
     body, pos = s._gen_struct_write( 'o', lhs, 's.'+mangled_lhs, dtype, 0 )
     assert pos == dtype.get_length()
 
-    upblk_content += body
+    upblk_content = body
     make_indent( upblk_content, 1 )
 
     blocks += upblk_content
@@ -1156,33 +1184,33 @@ m->{name}{sub} = {deference}model->{name}{sub};
 
   def _gen_ref_write( s, lhs, rhs, nbits, equal='=' ):
     if nbits <= 64:
-      return [ f"{lhs}[0] {equal} int({rhs})" ]
+      return [ '', f"{lhs}[0] {equal} int({rhs})" ]
     else:
-      ret = []
+      ret = [ '', f'x = {lhs}' ]
       ITEM_BITWIDTH = 32
       num_assigns = (nbits-1)//ITEM_BITWIDTH+1
       for idx in range(num_assigns):
         l = ITEM_BITWIDTH*idx
         r = l+ITEM_BITWIDTH if l+ITEM_BITWIDTH <= nbits else nbits
-        ret.append( f"{lhs}[{idx}] {equal} int({rhs}[{l}:{r}])" )
+        ret.append( f"x[{idx}] {equal} int({rhs}[{l}:{r}])" )
       return ret
 
   def _gen_ref_read( s, lhs, rhs, nbits, equal='=' ):
     if nbits <= 64:
-      return [ f"{lhs} {equal} Bits{nbits}({rhs}[0])" ]
+      return [ '', f"{lhs} {equal} {rhs}[0]" ]
     else:
-      ret = []
+      ret = [ '', f'x = {rhs}' ]
       ITEM_BITWIDTH = 32
       num_assigns = (nbits-1)//ITEM_BITWIDTH+1
       for idx in range(num_assigns):
         l = ITEM_BITWIDTH*idx
         r = l+ITEM_BITWIDTH if l+ITEM_BITWIDTH <= nbits else nbits
         _nbits = r - l
-        ret.append( f"{lhs}[{l}:{r}] {equal} Bits{_nbits}({rhs}[{idx}])" )
+        ret.append( f"{lhs}[{l}:{r}] @= x[{idx}]" )
       return ret
 
   def _gen_bits_decl( s, nbits ):
-    if nbits <= 256:
+    if nbits < 256:
       return f'Bits{nbits}'
     else:
       return f'mk_bits({nbits})'
